
    .data

      Alloc_pIMalloc dd 0

    .code

IMalloc proc public
;{
    ;--------------------------------------------------------------------------
    ; IMalloc returns the interface pointer to the memory allocator.
    ; This interface pointer cannot be used from a remote process -
    ; each process must have its own allocator.
    ;
    ; If an error occured while fetching the interface pointer,
    ; IMalloc returns NULL.
    ;
    ; EXAMPLE:
    ; invoke IMalloc ; returns interface pointer in eax
    ;
    ; Uses: eax, ecx, edx
    ;--------------------------------------------------------------------------

    lea     ecx, [ Alloc_pIMalloc ]
    mov     eax, [ ecx ]

    .IF eax
    ;{
        ret
    ;}
    .ENDIF

@@: invoke  CoGetMalloc, 1, ecx
    or      eax, eax
    jz      IMalloc
    ;{
        xor eax, eax
        ret
    ;}
;}
IMalloc endp

_Alloc proc public cb : DWORD
;{
    ;--------------------------------------------------------------------------
    ; The Alloc function uses the default allocator to allocate a memory block
    ; in the same way that malloc does. The initial contents of the returned
    ; memory block are undefined  there is no guarantee that the block has
    ; been initialized. The allocated block may be larger than cb bytes because
    ; of the space required for alignment and for maintenance information.
    ;
    ; If cb is zero, Alloc allocates a zero-length item and returns a valid
    ; pointer to that item. If there is insufficient memory available, Alloc
    ; returns NULL.
    ;
    ; Applications should always check the return value from this method, even
    ; when requesting small amounts of memory, because there is no guarantee
    ; the memory will be allocated.
    ;
    ; EXAMPLE:
    ; invoke Alloc, 128 ; allocates 128 bytes, returns memory pointer in eax
    ;
    ; Uses: eax, ecx, edx
    ;--------------------------------------------------------------------------

    invoke  IMalloc

    .IF eax
    ;{
        push    [ cb ]
        push    eax
        mov     eax, [ eax ]
        call    DWORD PTR [ eax + 12 ]
    ;}
    .ENDIF

    ret

_Alloc endp

Realloc proc public pv : DWORD, cb : DWORD
;{
    ;--------------------------------------------------------------------------
    ; The Realloc function changes the size of a memory block allocated by
    ; a previous call to Alloc or Realloc, in the same way that realloc does.
    ;
    ; The pv argument points to the beginning of the memory block.
    ; If pv is NULL, Realloc allocates a new memory block in the same way as
    ; the Alloc function. If pv is not NULL, it should be a pointer returned by
    ; a prior call to Alloc or Realloc.
    ;
    ; The cb argument specifies the size (in bytes) of the new block. The
    ; contents of the block are unchanged up to the shorter of the new and old
    ; sizes, although the new block can be in a different location. Because the
    ; new block can be in a different memory location, the pointer returned by
    ; Realloc is not guaranteed to be the pointer passed through the pv argument.
    ;
    ; If pv is not NULL and cb is zero, then the memory pointed to by pv is
    ; freed, with a return value of NULL.
    ;
    ; If pv is not NULL, cb is a valid size, and there was not enough memory
    ; available to expand the block to the given size, Realloc will return NULL,
    ; and the original block will remain unchanged.
    ;
    ; EXAMPLE:
    ; invoke Realloc, pv, 256 ; reallocates the memory at pv, pointer in eax
    ;
    ; Uses: eax, ecx, edx
    ;--------------------------------------------------------------------------

    invoke  IMalloc

    .IF eax
    ;{
        push    [ cb ]
        push    [ pv ]
        push    eax
        mov     eax, [ eax ]
        call    DWORD PTR [ eax + 16 ]
    ;}
    .ENDIF

    ret
;}
Realloc endp

GetSize proc public pv : DWORD
;{
    ;--------------------------------------------------------------------------
    ; The GetSize function returns the size of a memory block allocated by a
    ; previous call to Alloc or Realloc. If pv is a NULL pointer, the return
    ; value is -1.
    ;
    ; The size returned is the actual size of the allocation, which may be
    ; greater than the size requested when the allocation was made due to
    ; space required for alignment and for maintenance information.
    ;
    ; EXAMPLE:
    ; invoke GetSize, pv ; returns the memory block size (in bytes) in eax
    ;
    ; Uses: eax, ecx, edx
    ;--------------------------------------------------------------------------

    invoke  IMalloc

    .IF eax
    ;{
        push    [ pv ]
        push    eax
        mov     eax, [ eax ]
        call    DWORD PTR [ eax + 24 ]
    ;}
    .ENDIF

    ret
;}
GetSize endp

_Free proc public pv : DWORD
;{
    ;--------------------------------------------------------------------------
    ; The Free function, using the default OLE allocator, frees a block of memory
    ; previously allocated through a call to the Alloc or Realloc function.
    ;
    ; The number of bytes freed equals the number of bytes that were originally
    ; allocated or reallocated. After the call, the memory block pointed to by
    ; pv is invalid and can no longer be used.
    ;
    ; The pv parameter can be NULL. If so, this method has no effect.
    ;
    ; EXAMPLE:
    ; invoke Free, pv ; frees the memory at pv, doesn't return a value (void)
    ;
    ; Uses: eax, ecx, edx
    ;--------------------------------------------------------------------------

    invoke IMalloc

    .IF eax
    ;{
        push    [ pv ]
        push    eax
        mov     eax, [ eax ]
        call    DWORD PTR [ eax + 20 ]
    ;}
    .ENDIF

    ret
;}
_Free endp




